@@ -215,7 +215,7 @@ def postprocess_privilege_options(self, defaults=None):
 
     # Clean unused privilege starting with _ or .
     for k in list_unused_privilege(privileges, self['privilege_aliases']):
-        logger.debug("Drop unused hidden privilege %s", k)
+        logger.debug("Drop unused inactive privilege %s", k)
         del privileges[k]
 
     self['privileges'] = dict([
@@ -364,6 +364,7 @@ class Configuration(dict):
             'host': None,
             'port': None,
             'binddn': None,
+            'sasl_mech': None,
             'starttls': None,
             'user': None,
             'password': None,
@@ -375,6 +376,7 @@ class Configuration(dict):
             SELECT datname FROM pg_catalog.pg_database
             WHERE datallowconn IS TRUE ORDER BY 1;
             """),
+            'fallback_owner': None,  # None is current user.
             # SQL Query to inspect roles in cluster. See
             # https://www.postgresql.org/docs/current/static/view-pg-roles.html
             # and
@@ -426,6 +428,7 @@ class Configuration(dict):
         Mapping('ldap:host', env=None),
         Mapping('ldap:port', env=None),
         Mapping('ldap:starttls', env=None),
+        Mapping('ldap:sasl_mech', env=None),
         Mapping('ldap:binddn', env=None),
         Mapping('ldap:user', env=None),
         Mapping('ldap:password', secret=True, env=None),
@@ -435,10 +438,11 @@ class Configuration(dict):
             secret=r'(?:password=|:[^/][^/].*@)',
         ),
         Mapping('postgres:databases_query', env=None),
-        Mapping('postgres:owners_query', env=None),
-        Mapping('postgres:roles_query', env=None),
+        Mapping('postgres:fallback_owner', env=None),
         Mapping('postgres:managed_roles_query', env=None),
+        Mapping('postgres:owners_query', env=None),
         Mapping('postgres:roles_blacklist_query', env=None),
+        Mapping('postgres:roles_query', env=None),
         Mapping('postgres:schemas_query', env=None),
         Mapping(
             'postgres:shared_queries', processor=V.shared_queries, env=None),
@@ -496,7 +500,7 @@ class Configuration(dict):
 
     EPILOG = dedent("""\
 
-    ldap2pg requires a configuration file to describe LDAP queries and role
+    ldap2pg requires a configuration file to describe LDAP searches and role
     mappings. See https://ldap2pg.readthedocs.io/en/latest/ for further
     details.
 
