@@ -142,20 +142,37 @@ class Role(object):
                 )
             )
 
-    _drop_objects_sql = dedent("""
-    DO $$BEGIN EXECUTE 'GRANT "%(role)s" TO "'||SESSION_USER||'";'; END$$;
-    DO $$BEGIN
-      EXECUTE 'REASSIGN OWNED BY "%(role)s" TO "'||SESSION_USER||'";';
-    END$$;
-    DROP OWNED BY "%(role)s";
-    """)
-
-    def drop(self):
+    def drop(self, databases=None, me=None):
         yield Query(
-            'Reassign %s objects and purge ACL on %%(dbname)s.' % (self.name,),
-            Query.ALL_DATABASES,
-            self._drop_objects_sql % dict(role=self.name),
+            'Terminate running sessions for %s.' % self.name,
+            None, dedent("""\
+            SELECT pg_terminate_backend(pid)
+            FROM pg_catalog.pg_stat_activity
+            WHERE usename = '%s';
+            """) % self.name,
         )
+        databases = databases or []
+        for db in databases:
+            fmtkw = dict(owner=db.owner, role=self.name, me=me)
+            queries = []
+            if me in self.parents:
+                # Break membership loop before granting.
+                queries.append("""REVOKE "%(me)s" FROM "%(role)s";""")
+                self.parents.remove(me)
+
+            if me not in self.members:
+                # Inherit from target role to reassign even if non-superuser.
+                queries.append("""GRANT "%(role)s" TO %(me)s;""")
+                self.members.append(me)
+
+            queries.append("""REASSIGN OWNED BY "%(role)s" TO "%(owner)s";""")
+            queries.append("""DROP OWNED BY "%(role)s";""")
+
+            yield Query(
+                "Reassign %s's objects and purge ACL in %s." % (self.name, db),
+                db.name, '\n'.join(queries) % fmtkw,
+            )
+
         yield Query(
             'Drop %s.' % (self.name,),
             None,
@@ -276,8 +293,8 @@ class RoleSet(set):
     def resolve_membership(self):
         index_ = self.reindex()
         for role in self:
-            while role.parents:
-                parent_name = role.parents.pop()
+            # Synchronize role.parents -> parent.members
+            for parent_name in role.parents[:]:
                 try:
                     parent = index_[parent_name]
                 except KeyError:
@@ -286,6 +303,16 @@ class RoleSet(set):
                     continue
                 parent.members.append(role.name)
 
+            # Synchronize role.members -> member.parents
+            for member_name in role.members[:]:
+                try:
+                    member = index_[member_name]
+                except KeyError:
+                    raise ValueError('Unknown member role %s' % member_name)
+                if role.name in member.parents:
+                    continue
+                member.parents.append(role.name)
+
     def reindex(self):
         return dict([(role.name, role) for role in self])
 
@@ -318,7 +345,9 @@ class RoleSet(set):
     def union(self, other):
         return self.__class__(self | other)
 
-    def diff(self, other=None, available=None):
+    def diff(
+            self, other=None, available=None, fallback_owner=None,
+            databases=None, me=None):
         # Yield query so that self match other. It's kind of a three-way diff
         # since we reuse `available` roles instead of recreating roles.
 
@@ -426,8 +455,16 @@ class RoleSet(set):
 
         # Don't forget to trash all spurious managed roles!
         spurious = RoleSet(self - other - set(['public']))
+        # reassign databases to fallback_owner
+        for database in databases or []:
+            if database.owner in spurious:
+                for query in database.reassign(fallback_owner):
+                    yield query
+                # Update representation for the following queries.
+                database.owner = fallback_owner
+
         for role in reversed(list(spurious.flatten())):
-            for qry in role.drop():
+            for qry in role.drop(databases or [], me):
                 yield qry
 
 
